from django.db.models.fields import CharField, EmailField, TextField

class CIText: ...
class CICharField(CIText, CharField): ...
class CIEmailField(CIText, EmailField): ...
class CITextField(CIText, TextField): ...
