# Stubs for django.db.migrations (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from .migration import Migration as Migration
from .migration import swappable_dependency as swappable_dependency
from .operations import *
