from typing import Any

free: Any

def last_arg_byref(args: Any) -> Any: ...
def check_dbl(result: Any, func: Any, cargs: Any) -> Any: ...
def check_geom(result: Any, func: Any, cargs: Any) -> Any: ...
def check_minus_one(result: Any, func: Any, cargs: Any) -> Any: ...
def check_predicate(result: Any, func: Any, cargs: Any) -> Any: ...
def check_sized_string(result: Any, func: Any, cargs: Any) -> Any: ...
def check_string(result: Any, func: Any, cargs: Any) -> Any: ...
