GDAL_PIXEL_TYPES: dict[int, str]
GDAL_INTEGER_TYPES: list[int]
GDAL_TO_CTYPES: type
GDAL_RESAMPLE_ALGORITHMS: dict[str, int]
GDAL_COLOR_TYPES: dict[int, str]
VSI_FILESYSTEM_BASE_PATH: str
VSI_TAKE_BUFFER_OWNERSHIP: bool
VSI_DELETE_BUFFER_ON_READ: bool
