from typing import Any

def manager_repr(value: Any): ...
def smart_repr(value: Any): ...
def field_repr(field: Any, force_many: bool = ...): ...
def serializer_repr(serializer: Any, indent: Any, force_many: Any | None = ...): ...
def list_repr(serializer: Any, indent: Any): ...
