from collections.abc import Iterable, Iterator

from django.core.files.storage import Storage

def matches_patterns(path: str, patterns: Iterable[str]) -> bool: ...
def get_files(storage: Storage, ignore_patterns: Iterable[str] | None = ..., location: str = ...) -> Iterator[str]: ...
def check_settings(base_url: str | None = ...) -> None: ...
